#!/usr/bin/python

###############################################################################
# 
#    This file is part of COLLATINVS.
#
#    COLLATINVS is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    COLLATINVS is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with COLLATINVS; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###############################################################################

import string, re, sys

aide = """Collatinus pour python. Utilisation :
python collat.py -i : interrogation
python collat.py --aide : affiche cette aide ;
python collat.py --html fichier : analyse le fichier fichier, 
   et cre en sortie un fichier de mme nom mais portant 
   l'extension .html et donnant le texte et la lemmatisation
   de chaque mot classe alphabtiquement. 
python collat.py --latex fichier : mme chose, avec cration
   en sortie un fichier au format .tex.
"""
# --aide : affiche un texte d'aide
nb_arg = len(sys.argv)
if nb_arg < 2 or (nb_arg > 1 and sys.argv[1] == "--aide"): 
    sys.exit(aide) 
 
# =====================================================
# morphologie
def Nombre(i):
   Lnombres = [' ', 'singulier', 'pluriel']
   return Lnombres[i]
   
def Modele(i):  # ligne 1: 0-4; 2: 5-10; 3: 11-16; 4: 17-23; 5: 24-28; 6: 29,30
   Lmodeles = ['uita', 'amicus', 'puer', 'ager', 'templum', 
      'miles', 'ciuis', 'corpus', 'mare', 'manus', 'res',      
      'bonus', 'miser', 'pulcher', 'fortis', 'uetus', 'acer',
      'amo', 'moneo', 'lego', 'capio', 'audio', 'sum', 'eo',
      'imitor', 'uereor', 'sequor', 'patior', 'potior',
      'pronoms', 'invaria']
   return Lmodeles[i]
   
def MorphoK(m):
   if m < 11:
      return "nominatif singulier"
   elif m < 17:
      return "nominatif masculin singulier"
   elif m < 29:
      return "1re singulier prsent indicatif actif"
   else: 
      return ""
      
def DesK(m):
   if m == 0:
      return 'a'
   elif m in [1, 9, 11]: 
      return 'us'
   elif 1 < m < 4 or 11 < m < 14:
      return 'er'
   elif m == 4:
      return 'um'
   elif m == 10:
      return 'es'
   elif m == 17 or m == 19:
      return 'o'
   elif m == 18 or m == 23:
      return 'eo'
   elif 19 < m < 22:
      return 'io'
   elif m == 22:
      return 'sum'
   elif m == 24 or m == 26:
      return 'or'
   elif m == 25:
      return 'eor'
   else:
      return 'ior'
   
def Cas(i):   
   Lcas = ['', 'nominatif', 'vocatif', 'accusatif', 'gnitif', 'datif', 'ablatif']
   return Lcas[i]

def Genre(i):
   Lgenres = ['', 'masculin', 'fminin', 'neutre']
   return Lgenres[i]
   
def Personne(i):
   Lpersonnes = ['', '1re', '2me', '3me']
   return Lpersonnes[i]
   
def Temps(i):
   Ltemps = ['', 'prsent', 'futur', 'imparfait', 'parfait',
      'futur antrieur', 'plus-que-parfait']
   return Ltemps[i]
   
def Mode(i):
   Lmodes = ['', 'indicatif', 'subjonctif', 'impratif', 'infinitif',
     'participe', 'grondif',  'adjectif verbal']
   return Lmodes[i]
   
def Voix(i):
   Lvoix = ['', 'actif', 'passif']
   return Lvoix[i]
   
# =====================================================   
   
# =====================================================
# classe tdes
class tdes:   
   def __init__(self, gr, c, gn, n, d, p, t, mde, v, mdl, r):
      self.graphie = gr
      self.cas = int(c)
      self.genre = int(gn)
      self.nombre = int(n)
      self.degre = int(d)
      self.personne = int(p)
      self.temps = int(t)
      self.mode = int(mde)
      self.voix = int(v)
      self.modele = int(mdl)
      self.radnum = int(r)
         
   def morpho(self):  
      """retourne la morphologie de l'instance en utilisant les listes ci-dessus. """
      return '%s %s %s %s %s %s %s (%s) ' % (Cas(self.cas), 
         Genre(self.genre), Personne(self.personne), 
         Nombre(self.nombre), Temps(self.temps), Mode(self.mode), 
         Voix(self.voix), Modele(self.modele))   
         
   def affiche(self):
      print self.morpho()
            
# =====================================================

# =====================================================
#~ classe tirregs
class tirreg(tdes):
   def __init__(self, gr, k, c, gn, n, p, t, mde, v, mdl):
      self.graphie = gr
      self.cas = int(c)
      self.genre = int(gn)
      self.nombre = int(n)
      self.personne = int(p)
      self.temps = int(t)
      self.mode = int(mde)
      self.voix = int(v)
      self.modele = int(mdl)
      self.canon = k

# =====================================================

# =====================================================
class tentree:
   def __init__(self, m, g, t, r2, r3):
      self.modele = int(m)
      self.graphie = g
      self.texte = t
      #~ intgration des radicaux...

   def doc(self):
      """ retourne la forme canonique suivie de ses donnes morphologiques, et des traductions franaises """
      return '<b>%s</b>, %s' % (self.graphie, self.texte)

   def doctex(self):
      """ comme doc, mais en LaTeX """
      return '\\textbf{%s}, %s' % (self.graphie, self.texte)   

   def affiche(self):
      print self.doc()
      
# =====================================================

# =====================================================
class tradical:  
   def __init__(self, g, k, m):
      self.graphie = g
      self.canon = k
      self.modele = m
    
   def kanon(self):
       """ retourne l'entre de lexique correspondant au radical """  
       return lexique[self.canon]
         
   def affiche(self, i):
      print "%s : radical %d de %s" % (self.graphie, i, self.canon)

# =====================================================

# =====================================================
# chargement du lexique

print "Collatinus pour python\n  chargement du lexique..."

#   lecture xml      
def decrasse(l, balise):
    """ ridicule parseur xml en trois lignes : 
        dbarasse une ligne de <balise> et de </balise>"""
    r = string.replace(l, '<%s>' % (balise), '')
    r = string.replace(r, '</%s>' % (balise), '')
    return string.strip(r)

#~ 1. entres

fichier = open('canons.xml')
ligne = ""
lexique = {}
radicaux = {}
radicaux[2] = {}
radicaux[3] = {}
m = 0
t = g = r2 = r3 = ""

# passer l'en-tte et les commentaires
while string.find(ligne, "<collat") < 0:
   ligne = fichier.readline()

# lecture et parsage   
while string.find(ligne, "</collat") < 0:
   ligne = fichier.readline()
   if string.find(ligne, "<modele>") > -1:
      m = decrasse(ligne, "modele")
   elif string.find(ligne, "<graphie>") > -1:
      g = decrasse(ligne, "graphie")
   elif string.find(ligne, "<texte>") > -1:
      t = decrasse(ligne, "texte")
   elif string.find(ligne, "<R2>") > 1:
      r2 = decrasse(ligne, "R2")
   elif string.find(ligne, "<R3>") > 1:
      r3 = decrasse(ligne, "R3")
   elif string.find(ligne, "</canon>") > -1:
      if lexique.has_key(g):
         g = g + ' (2)'
      lexique[g] = tentree(int(m), g, t, r2, r3) 
      if r2 > "":
         if not radicaux[2].has_key(r2):
            radicaux[2][r2] = {}
            radicaux[2][r2][0] = tradical(r2, g, int(m))
         else:
            radicaux[2][r2][len(radicaux[2][r2])] = tradical(r2, g, int(m))
         r2 = ""
      if r3 > "":
         if not radicaux[3].has_key(r3):
            radicaux[3][r3] = {}
            radicaux[3][r3][0] = tradical(r3, g, int(m))
         else:         
            radicaux[3][r3][len(radicaux[3][r3])] = tradical(r3, g, int(m))
         r3 = ""
    
fichier.close()  

#~ 2. dsinences

print "  chargement des desinences..."

desinences = {}

fichier = open('desinences.xml')

# passer l'en-tte et les commentaires
while string.find(ligne, "<collat") < 0:
   ligne = fichier.readline()

#~ initialisation de la premire dsinence
ligne = '';
cas = genre = nombre = degre = personne = mode = temps\
   = voix = modele = radnum = '0'
#~ boucle de lecture
while string.find(ligne, "</collat") < 0:
   ligne = fichier.readline()   
   if string.find(ligne, "<graphie>") > -1:
      g = decrasse(ligne, "graphie")
   elif string.find(ligne, "cas") > -1:
      cas = decrasse(ligne, "cas")
   elif string.find(ligne, "<genre>") > -1:
      genre = decrasse(ligne, "genre")
   elif string.find(ligne, "<nombre>") > -1:
      nombre = decrasse(ligne, "nombre")
   elif string.find(ligne, "<degre>") > -1:
      degre = decrasse(ligne, "degre")
   elif string.find(ligne, "personne>") > -1:
      personne = decrasse(ligne, "personne")
   elif string.find(ligne, "<temps>") > -1:
      temps = decrasse(ligne, "temps")
   elif string.find(ligne, "<mode>") > -1:
      mode = decrasse(ligne, "mode")
   elif string.find(ligne, "<voix>") > -1:
      voix = decrasse(ligne, "voix")
   elif string.find(ligne, "<modele>") > -1:
      modele = decrasse(ligne, "modele")
   elif string.find(ligne, "<R>") > -1:
      radnum = decrasse(ligne, "R")
   elif string.find(ligne, "</desinence>") > -1:
    #~ def __init__(self, gr, c, gn, n, d, p, t, mde, v, mdl, r):
      des = tdes(g, cas, genre, nombre, degre, personne,
         temps, mode, voix, modele, radnum)
      if not desinences.has_key(g):
         #~ ajout de la cl et de la dsinence
         desinences[g] = {}
         desinences[g][0] = des
      #~ ajout de la dsinence dans la cl      
      else :
         desinences[g][len(desinences[g])] = des
      #~ rinitialisation graphie et modele ont toujours une valeur   
      cas = genre = nombre = degre = personne = temps = mode = voix = '0'
      
fichier.close()   

#~ irrguliers
print "  chargement des formes irregulieres..."
irregs = {}
fichier = open("irregs.xml")

# passer l'en-tte et les commentaires
while string.find(ligne, "<collat") < 0:
   ligne = fichier.readline()

ligne = '';
cas = genre = nombre = degre = personne\
    = mode = temps = voix = modele = '0'

#~ boucle de lecture
while string.find(ligne, "</collat") < 0:
   ligne = fichier.readline()   
   if string.find(ligne, "<graphie>") > -1:
      g = decrasse(ligne, "graphie")
   elif string.find(ligne, "cas") > -1:
      cas = decrasse(ligne, "cas")
   elif string.find(ligne, "<genre>") > -1:
      genre = decrasse(ligne, "genre")
   elif string.find(ligne, "<nombre>") > -1:
      nombre = decrasse(ligne, "nombre")
   elif string.find(ligne, "<degre>") > -1:
      degre = decrasse(ligne, "degre")
   elif string.find(ligne, "personne>") > -1:
      personne = decrasse(ligne, "personne")
   elif string.find(ligne, "<temps>") > -1:
      temps = decrasse(ligne, "temps")
   elif string.find(ligne, "<mode>") > -1:
      mode = decrasse(ligne, "mode")
   elif string.find(ligne, "<voix>") > -1:
      voix = decrasse(ligne, "voix")
   elif string.find(ligne, "<modele>") > -1:
      modele = decrasse(ligne, "modele")
   elif string.find(ligne, "<canon>") > -1:
      canon = decrasse(ligne, "canon")
   elif string.find(ligne, "</irreg>") > -1:
    #~ irreg __init__(self, gr, k, c, gn, n, p, t, mde, v, mdl)
      irreg = tirreg(g, canon, cas, genre, nombre, personne,
         temps, mode, voix, modele)
      if not irregs.has_key(g):
         #~ ajout de la cl et de la dsinence
         irregs[g] = {}
         irregs[g][0] = irreg
      #~ ajout de la dsinence dans la cl      
      else:
         irregs[g][len(irregs[g])] = irreg
      #~ rinitialisation graphie et modele ont toujours une valeur   
      cas = genre = nombre = degre = personne = temps = mode = voix = '0'
      
fichier.close()

print "  ", len(lexique), " entrees; ",
print len(desinences), " desinences;\n  ",
print len(radicaux[2]), "radicaux 2; ",
print len(radicaux[3]), "radicaux 3; ",
print len(irregs), "irreguliers.\n",

# =====================================================
# consultation du lexique 
def dico():
   m = "dico"
   while m != "q":
      if m > "":
         if lexique.has_key(m):
            lexique[m].affiche()
         else:
            print "introuvable"
         m = raw_input("entre (q pour quitter) : ")
# ===================================================== 

# =====================================================
# consultation des dsinences
def dicodes():
   mot = ""
   while mot != "q":
      if mot != "":   
         if desinences.has_key(mot):
            for i in range(len(desinences[mot])):
               desinences[mot][i].affiche()
         else:
            print "introuvable"           
      mot = raw_input("dsinence ? (q pour quitter) ")
# =====================================================

# =====================================================
#~ consultation des radicaux
def dicorad():
   mot = ""
   while mot != "q":
      if mot > "":
         for i in [2, 3]:
            if radicaux[i].has_key(mot):
               for iRad in range(len(radicaux[i][mot])):
                  radicaux[i][mot][iRad].affiche(i)
      mot = raw_input("radical ? (q pour quitter) ")      

# =====================================================
# recherche des dsinences possibles d'un mot
def desforme():
   mot = ""
   while not mot == "q":
      if mot > '':
         for i in range(len(mot)):
            if desinences.has_key(mot[i:]):
               Des = desinences[mot[i:]]
               for iMorph in range(len(Des)):
                   print Des[iMorph].graphie,
                   Des[iMorph].affiche()
      mot = raw_input("forme ? (q pour quitter) ")
# =====================================================

# =====================================================
# analyses morphologiques d'un mot
def analysesDe(mot):
   resultat = {}
   #~ chercher les formes canoniques
   if lexique.has_key(mot):
      resultat[mot] = []
      resultat[mot].append(MorphoK(lexique[mot].modele))
   #~ chercher les formes irrgulires
   if irregs.has_key(mot):
      for iIrr in range(len(irregs[mot])):
         Irr = irregs[mot][iIrr]
         if not resultat.has_key(Irr.canon):
            resultat[Irr.canon] = []
         resultat[Irr.canon].append(Irr.morpho())   
   # chercher toutes les dsinences
   for i in range(len(mot)):
      grDes = mot[i:]
      grRad = mot[:i]
      if desinences.has_key(grDes):
         lesDes = desinences[grDes]
         for iDes in range(len(lesDes)):
            Des = lesDes[iDes]
            grMorpho = Des.morpho()
            if Des.radnum > 1:
               if radicaux[Des.radnum].has_key(grRad):
                  radix = radicaux[Des.radnum][grRad]
                  for iRad in range(len(radix)):
                     if Des.modele == radix[iRad].modele and lexique.has_key(radix[iRad].canon)\
                        and  radix[iRad].modele == lexique[radix[iRad].canon].modele:
                     #~ si la cl n'existe pas dans resultat
                        if not resultat.has_key(radix[iRad].canon):
                            resultat[radix[iRad].canon] = []
                            resultat[radix[iRad].canon].append(grMorpho)                         
                        else:
                           resultat[radix[iRad].canon].append(grMorpho)
            else: # if Des[iDes].modele > 13:
               K = grRad + DesK(Des.modele)     
               if lexique.has_key(K)\
                  and Des.modele == lexique[K].modele:
                     if not resultat.has_key(K):
                        resultat[K] = []     
                        resultat[K].append(grMorpho)
                     else: 
                        resultat[K].append(grMorpho)
   return resultat

# =====================================================
#~ chane de retour d'une analyse

def ijuv(forme):
   # transformation v-u et i-J
   r = string.replace(forme, 'v', 'u')
   r = string.replace(r, 'j', 'i')
   return r

def chaineAnalyse(forme):
    global finem
    finem = forme == "augustulis"
    f = ijuv(forme)
    res = analysesDe(f)
    if res == {}:
        return "?"
    else  :
        retour = ""
        for i in range(len(res)):
            K = res.keys()[i]
            retour = retour + lexique[K].doc() + '\n'
            for iK in range(len(res[K])):
                retour = retour + '    '+ res[K][iK] + '\n'
        return retour

# =====================================================
# dialogue d'analyse morphologique

def dialogueAnalyse():
   forme = ""
   while forme != "q":
      print chaineAnalyse(forme)  
      forme = raw_input("forme ? (q pour quitter) ")

# =====================================================
# lemmatisation d'un texte, retour en html.

def lemmatise(t):
   """renvoie dans un tableau html de deux colonnes
      le lexique utilis par le texte t."""
   # equivalences j->i et v->u
   t = ijuv(t)
   # requete d'analyse morpho :
   if t[0:2] == 'm:': 
          t = t[2:]
          return analyseTotale(t)
   # suppression de la ponctuation et du texte entre [], mise en tableau
   t = re.sub('\[.*\]','', t)
   # ajout d'espaces en dbut et fin pour assurer la mise en tableau
   # des premier et dernier mots.
   t = " " + t + " "
   tableau = re.split('[^a-zA-Z]', t)
   retour = '<table align="center" width="100%"><tr><td width="50%" valign="top">\n'
   dico = []
   # lemmatisation de chaque mot
   for iTab in range(len(tableau)):
          f = tableau[iTab]
          lignes = analysesDe(f)
          if lignes == {}:              
             # essayer en minuscules
             lignes = analysesDe(ijuv(string.lower(f)))
          # essais suffixes
          if f[-3:] == 'que':
             lignes.update(analysesDe(f[:-3]))
          if lignes == {}:
             L = '<b>' + f + '</b> : ?'
             try:
                dico.index(L)
             except:
                dico.append(L)
          else:
             for iLin in range(len(lignes)):
                L = lexique[lignes.keys()[iLin]].doc()
                try:
                   dico.index(L)
                except:
                   dico.append(L)
   # tri et ajout au rsultat
   dico.sort()
   # eliminer le : ? en première ligne.
   del dico[0]
   for i in range(len(dico)):
         retour = retour + dico[i] + '<br />\n'
         if i == len(dico) / 2 :
            retour = retour + '</td><td valign="top">'
   retour = retour + '</td></tr></table>\n'
   return retour

# =====================================================
# lemmatisation d'un texte, retour en LaTeX.

def latexatise(t):
   """renvoie dans un tableau html de deux colonnes
      le lexique utilis par le texte t."""
   # quivalences j->i et v->u
   t = ijuv(t)
   # suppression de la ponctuation et du texte entre [], mise en tableau
   t = re.sub('\[.*\]','', t)
   # ajout d'espaces en dbut et fin pour assurer la mise en tableau
   # des premier et dernier mots.
   t = " " + t + " "
   tableau = re.split('[^a-zA-Z]', t)
   dico = []
   retour = ""
   # lemmatisation de chaque mot
   for iTab in range(len(tableau)):
          f = tableau[iTab]
          lignes = analysesDe(f)
          if lignes == {}:              
             # essayer en minuscules
             lignes = analysesDe(ijuv(string.lower(f)))
          # essais suffixes
          if f[-3:] == 'que':
             lignes.update(analysesDe(f[:-3]))
          if lignes == {}:
             L = '\\textbf{' + f + '} : ?\\'
             try:
                dico.index(L)
             except:
                dico.append(L)
          else:
             for iLin in range(len(lignes)):
                L = lexique[lignes.keys()[iLin]].doctex()
                try:
                   dico.index(L)
                except:
                   dico.append(L)
   # tri et ajout au rsultat
   dico.sort()
   # liminer le : ? en première ligne.
   del dico[0]
   for i in range(len(dico)):
         retour = retour + dico[i] + '\\\\\n'
   return retour

def changeext(nom, suffixe):
   import os
   eclats = os.path.splitext(nom)
   return eclats[0] + suffixe

#~ test du module 
if __name__ == '__main__':
    # examiner les paramtres :
    if sys.argv[1] == '-i':
 	dialogueAnalyse()       
    # --html infile fresultat : parse infile, et crit le rsultat html dans fresultat.
    elif sys.argv[1] == "--html":
        fichier = open(sys.argv[2], "r")
        t = fichier.read()
        sortie = '<html><body>' + t + lemmatise(t) + '</body></html>'
        fichier.close
        argv2 = changeext(sys.argv[2], '.html')
        fichier = open(argv2, "w");
        fichier.write(sortie);
        fichier.close;       	
    # --latex infile fresultat : parse infile,
    # et crit le rsultat latex dans fresultat
    elif sys.argv[1] == "--latex":
        entete = """\documentclass[12pt]{article}
	\usepackage[T1]{fontenc}
	\usepackage[latin1]{inputenc}
	\usepackage{geometry}
	\usepackage[frenchb]{babel}
	\usepackage{multicol}
	\geometry{a4paper, left=1in, right=1in, top=1in, bottom=1in}
	\\begin{document} """
        fichier = open(sys.argv[2], "r")
        t = fichier.read()
        sortie = entete + t
        sortie += "\\begin{multicols}{2}\n"
        sortie += latexatise(t)
        sortie += "\end{multicols}\n" 
        sortie += "\end{document}"
        fichier.close
        # changer l'extension
        argv2 = changeext(sys.argv[2], '.tex')
        fichier = open(argv2, "w");
        fichier.write(sortie);
        fichier.close;       	
